package org.openscm.kundo.plugins

/*
 * Copyright (C) 2008 The Ultimate People Company Ltd ("UPCO").
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import groovy.xml.NamespaceBuilder
import org.openscm.kundo.plugins.context.BuildContext
import org.openscm.kundo.common.logging.SimpleLogger

 /**
 * Project Delegate
 * @author <a href="mailto:ben.leedham@upco.co.uk">Ben Leedham</a>
 * @version 1.0.0
 * 
 * Description: Delegate to load various global properties for use by other plugins
 * and optionally parse a maven pom file.
 * 
 * <p>The project delegate reads indexed properties from either the default plugin.properties file
 * or the project's build.properties file. The properties with the same index are grouped allowing
 * output and source directories to be linked along with exclude and include patterns. If a Maven
 * pom file exists the project artifact information will be taken from it, if not then the project's
 * artifactId will be the name of the ant project. </p>
 */
/* 
 * @ant-target doProject
 */
class ProjectDelegate extends AbstractPluginTargetDelegate {

	private def mvn
	
	/*
	 * The location of the project's pom file if indeed there is one
	 * @ant-property-name project.pom.location
	 */	
	String pomLocation
	/*
	 * A set of source files grouped by key and index. Format: [plugin].[key].[index].[fieldname]
	 * @indexed-ant-property indexed
	 */	
	Map indexedProps
	
	/**
	 * Constructor sets ant and buildContext instances in super class
	 * @param ant			AntBuilder instance
	 * @param buildContext	BuildContext instance
	 */
	ProjectDelegate( AntBuilder ant, BuildContext buildContext ){
		super( ant, buildContext)
		this.mvn = NamespaceBuilder.newInstance( ant, 'antlib:org.apache.maven.artifact.ant' )
	}
	
	/**
	 * Implementation of abstract method from AbstractPluginTargetDelegate
	 * to hold assertions that all global and local properties are available to the plugin.
	 * This method is automatically called prior to execution.
	 */	
	void doCheck(){
		assert pomLocation != null
		assert indexedProps != null
		// assert global variables exists in ant project
		// there are no global variables in this plugin
	}	
	
	/**
	 * Setter method for member variable pomLocation
	 * @param pomLocation	String representation the project's pom location
	 */	
	void setPomLocation( String pomLocation ){
		this.pomLocation = pomLocation
	}
	
	/**
	 * Setter method for member variable indexedProps
	 * @param indexedProps	Map of properties grouoed into dynamic objects by index
	 */		
	void setIndexedProps( Map indexedProps ){
		this.indexedProps = indexedProps
	}	


	/**
	 * indexedProps is a Map that contains lists of properties specified with an index in a properties file (see plugin.properties)
	 * properties with the same key and index are grouped into objects by PropertyUtils.getPrefixedIndexedProperties()
	 * indexedProps is injected at runtime by the PluginTargetDelegateFactory
	 * we now need to set it as a reference in the ant project to allow other plugins to access it
	 */
	void doProject() {
		if( log.isDebugEnabled() ){
			log.debug( "Entering +doProject()" )
		}

		ant.project.addReference( "global.project.properties", indexedProps )		

		if( log.isDebugEnabled() ){
			log.debug( "Exiting +doProject() set ant project reference \"global.project.properties\" to $indexedProps" )
		}		
	}
	
	/**
	 * reads the pom location variable. If a pom exists then project info is harvested from it
	 * if not then the project artifactId is that of the ant project name
	 *
	 */
	void doPomProperties(){
		if( log.isDebugEnabled() ){
			log.debug( "Entering +doPomProperties()" )
		}	
	
		def groupId
		def artifactId
		def version
		def packaging
	
		File pomFile = new File( pomLocation )
		if( pomFile.exists() ){
			if( log.isDebugEnabled() ){
				log.debug( "+doPomProperties() pom file exists at location: $pomLocation" )
			}
			
			ant.project.setProperty( "global.pom.location", pomLocation )
			
			mvn.pom( file:pomLocation, id:"global.maven.project" )
			groupId = buildContext.get( "maven.project.groupId" )
			artifactId = buildContext.get( "maven.project.artifactId" )
			version = buildContext.get( "maven.project.version" )
			packaging = buildContext.get( "maven.project.packaging" )
			
			ant.project.setProperty( "global.artifactid", artifactId )
			ant.project.setProperty( "global.groupid", groupId )
			ant.project.setProperty( "global.version", version )
			ant.project.setProperty( "global.packaging", packaging )			

		}else{			
			artifactId = ant.project.getName()
			ant.project.setProperty( "global.artifactid", artifactId )
		}
		
		if( log.isDebugEnabled() ){
			log.debug( "Exiting +doPomProperties()" )
		}		
	}
	
	/**
	 * Concrete implementation of the AbstractPluginTargetDelegate.execute() method
	 */
	void doExecute(){
		doProject()
		doPomProperties()
	}
	
}